from functools import wraps


def repeat(loop):
    def __inner(fn):
        @wraps(fn)
        def ___inner(*args, **kwargs):
            ret = None
            for i in range(0, loop):
                ret = fn(*args, **kwargs)
            return ret
        return ___inner
    return __inner





import jinja2
import os

class Config(dict):
    """Dictionary like helper class for maintaining test data configurations per environment.

:class:`holmium.core.TestCase` and :class:`holmium.core.HolmiumNose` both
look for either a config.json or config.py file in the same directory as the
test file, and will make a ``config`` object available to the test case instance.

The :class:`holmium.core.Config` object is aware of the environment (specified with ``--holmium-env``
when using nose or ``HO_ENV`` as an environment variable and will return the config variable from that
environment or from the `default` key.

Values in the config file can use :class:`jinja2.Template` templates to access either values from itself, environment variables
or a select magic holmium variables: ``holmium.environment``, ``holmium.browser``, ``holmium.user_agent``
and ``holmium.remote``.

Example config structure (which uses a magic variable ``holmium.environment`` and an environment variable ``$PATH``).

JSON

.. code-block:: json

    {
            'default': { 'url': 'http://localhost'
                        , 'path':"{{PATH}}"
                        , 'login_url': '{{default.url}}/{{holmium.environment}}/login'}
                        , 'username' : '{{holmium.environment}}user'}
            ,'production': {'password': 'sekret'}
            ,'development': {'password': 'password'}
    }

Python

.. code-block:: python

    config = {
        {
            'default': { 'url': 'http://localhost'
                        , 'path':"{{PATH}}"
                        , 'login_url': '{{default.url}}/{{holmium.environment}}/login'}
                        , 'username' : '{{holmium.environment}}user'}
            ,'production': {'password': 'sekret'}
            ,'development': {'password': 'password'}
        }
    }

When accessing ``self.config`` within a test, due to the default:

* ``self.config['path']`` will always return the value of the environment variable `PATH`,
* ``self.config['password']`` will always return 'sekret'
* ``self.config['url']`` will always return 'http://localhost'

if ``HO_ENV`` or ``--holmium-env`` are ``production``:

* ``self.config['username']`` will return ``productionuser``
* ``self.config['password']`` will return ``sekret``
* ``self.config['login_url']`` will return ``http://localhost/production/login``

if ``HO_ENV`` or ``--holmium-env`` are ``development``:

* ``self.config['username']`` will return ``developmentuser``
* ``self.config['password']`` will return ``password``
* ``self.config['login_url']`` will return ``http://localhost/development/login``

    """
    def __init__(self, dct, environment={"holmium":{"environment":"development"}}):
        self.env = environment
        dict.__init__(self, dct)

    def __getitem__(self, key):
        def __render(item, context):
            if issubclass(item.__class__, str) or issubclass(item.__class__,unicode):
                template = jinja2.Template(item)
                rendered = template.render(context)
                if rendered != item:
                    return __render(rendered,context)
                else:
                    return rendered
            else:
                return item
        try:
            item = dict.__getitem__(self, self.env["holmium"]["environment"])[key]
        except KeyError,e:
            item = dict.__getitem__(self, "default")[key]

        context = dict(self)
        context.update(os.environ)
        context.update(self.env)
        return __render(item, context)

    def __setitem__(self, key, value):
        sub_dict = dict.setdefault(self, self.env["holmium"]["environment"], {})
        sub_dict[key] = value




from selenium.common.exceptions import NoSuchElementException, TimeoutException, NoSuchFrameException
from selenium.webdriver.support.ui import WebDriverWait, Select
from selenium.webdriver.remote.webelement import WebElement
import selenium.webdriver.common.by
import holmium
import inspect
import weakref
import types
import threading
import contextlib
import collections

try:
    from ordereddict import OrderedDict
except ImportError:
    from collections import OrderedDict

from functools import wraps
class Locators(selenium.webdriver.common.by.By):
    """
    proxy class to access locator types
    """
    pass


def enhanced (web_element):
    """
    incase a higher level abstraction for a WebElement is available
    we will use that in Pages. (e.g. a select element is converted into
    :class:`selenium.webdriver.support.ui.Select`)
    """
    abstraction_mapping = {'select': Select}
    if web_element.tag_name in abstraction_mapping.keys():
        return abstraction_mapping[web_element.tag_name](web_element)
    return web_element


class ElementList(list):
    """
    proxy to a standard list which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        list.__init__(self, *args, **kwargs)

    def __getitem__(self, index):
        return list.__getitem__(self, index).__get__(self.instance(),
                                                     self.instance().__class__)

class ElementDict(dict):
    """
    proxy to a standard dict which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        dict.__init__(self, *args, **kwargs)

    def __getitem__(self, key):
        return dict.__getitem__(self, key).__get__(self.instance(),
                                                   self.instance().__class__)


class Page(object):
    """
    Base class for all page objects to extend from.
    void Instance methods implemented by subclasses are provisioned
    with fluent wrappers to facilitate with writing code such as::

        class Google(Page):
            def enter_query(self):
                ....

            def submit_search(self):
                ....

            def get_results(self):
                ....

        assert len(Google().enter_query("page objects").submit_search().get_results()) > 0

    """
    local = threading.local()
    def __init__(self, driver, url=None, iframe=None):
        self.driver = driver
        if url:
            self.home = url
        elif driver.current_url:
            self.home = driver.current_url
        self.iframe = iframe
        for el in inspect.getmembers(self.__class__):
            def update_element(el):
                if issubclass(el.__class__, ElementGetter):
                    el.iframe = self.iframe

            if issubclass(el[1].__class__, list):
                for item in el[1]:
                    update_element(item)
                self.__setattr__(el[0], ElementList(self, el[1]))
            elif issubclass(el[1].__class__, dict):
                for item in el[1].values():
                    update_element(item)
                self.__setattr__(el[0], ElementDict(self, el[1]))
            else:
                update_element(el[1])

        if url:
            self.driver.get(url)

    @contextlib.contextmanager
    def scope(self):
        Page.local.driver = object.__getattribute__(self, "driver")
        yield

    @staticmethod
    def get_driver():
        return Page.local.driver

    def go_home(self):
        """
        returns the page object to the url it was initialized with
        """
        self.driver.get(self.home)

    def __getattribute__(self, key):
        """
        to enable fluent access to page objects, instance methods that
        don't return a value, instead return the page object instance.
        """
        with object.__getattribute__(self, "scope")():
            attr = object.__getattribute__(self, key)
            # check if home url is set, else update.
            if not object.__getattribute__(self, "home"):
                holmium.core.log.debug("home url not set, attempting to update.")
                object.__setattr__(self, "home", object.__getattribute__(self,"driver").current_url)

            if isinstance(attr, types.MethodType):
                @wraps(attr)
                def wrap(*args, **kwargs):
                    resp = attr(*args, **kwargs)
                    if not resp:
                        holmium.core.log.debug("method %s returned None, using fluent response" % attr.func_name)
                        resp = self
                    return resp
                return wrap
            return attr




class ElementGetter(object):
    """
    internal class to encapsulate the logic used by
    :class:`holmium.core.Element`
    & :class:`holmium.core.Elements`
    """

    def __init__(self, locator_type, query_string, base_element=None, timeout=1, value=lambda el:el):
        self.query_string = query_string
        self.locator_type = locator_type
        self.timeout = timeout
        self.driver = None
        self.iframe = None
        self.base_element = base_element
        self.value_mapper = value
        self.root_fn = lambda:Page.get_driver()
        holmium.core.log.debug("locator:%s, query_string:%s, timeout:%d" %
                              (locator_type, query_string, timeout))
    
    @property
    def root(self):
        return self.root_fn()

    def get_element(self, method = None):

        if self.base_element:
            if isinstance(self.base_element, types.LambdaType):
                el = self.base_element()
                _meth = getattr(el, method.im_func.func_name)
            elif isinstance(self.base_element, Element):
                _meth = getattr(self.base_element.__get__(self, self.__class__), method.im_func.func_name)
            elif isinstance(self.base_element, WebElement):
                _meth = getattr(self.base_element, "find_element")
            else:
                holmium.core.log.error("unknown base_element type used: %s" % type(self.base_element))
        else:
            _meth = method
        holmium.core.log.debug("looking up locator:%s, query_string:%s, timeout:%d" %
                              (self.locator_type, self.query_string, self.timeout))

        if self.iframe:
            Page.local.driver.switch_to_default_content()
            Page.local.driver.switch_to_frame(self.iframe)

        if self.timeout:
            try:
                WebDriverWait(self.root, self.timeout).until(lambda _: _meth(self.locator_type, self.query_string))
            except TimeoutException:
                holmium.core.log.debug("unable to find element %s after waiting for %d seconds" % (self.query_string, self.timeout))
        return _meth(self.locator_type, self.query_string)


class Element(ElementGetter):
    """
    Utility class to get a :class:`selenium.webdriver.remote.webelement.WebElement`
    by querying via one of :class:`holmium.core.Locators`
    """

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return self.value_mapper(enhanced(self.get_element(self.root.find_element)))
        except NoSuchElementException:
            return None

class Elements(ElementGetter):
    """
    Utility class to get a collection of :class:`selenium.webdriver.remote.webelement.WebElement`
    objects by querying via one of :class:`holmium.core.Locators`
    """

    def __getitem__(self, idx):
        return lambda: self.__get__(self, self.__class__)[idx]

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return [self.value_mapper(enhanced(el)) for el in self.get_element(self.root.find_elements)]
        except NoSuchElementException:
            return []


class ElementMap(Elements):
    """
    Used to create dynamic dictionaries based on an element locator specified by one of
    :class:`holmium.core.Locators`.

    The wrapped dictionary is an :class:`collections.OrderedDict` instance.

    :param lambda key: transform function for mapping a key to a WebElement in the collection
    :param lambda value: transform function for the value when accessed via the key.
    """
    def __init__(self, locator_type, query_string=None, base_element=None, timeout =1
            , key  = lambda el:el.text, value = lambda el:el):
        Elements.__init__(self, locator_type, query_string, base_element, timeout)
        self.key_mapper = key
        self.value_mapper = value

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return OrderedDict((self.key_mapper(el), self.value_mapper(enhanced(el))) for el in self.get_element(self.root.find_elements))
        except NoSuchElementException:
            return {}


    def __getitem__(self, key):
        return lambda:self.__get__(self, self.__class__)[key]

class Section(object):
    """
    Base class to encapsulate reusable page sections::

        class MySection(Section):
            things = Elements( .... )

        class MyPage(Page):
            section_1 =  MySection(Locators.CLASS_NAME, "section")
            section_2 =  MySection(Locators.ID, "unique_section")

    """
    def __init__(self, locator_type, query_string, iframe=None):
        self.locator_type = locator_type
        self.query_string = query_string
        self.iframe = iframe
        self.__root_val = None
    def __get__(self, instance, owner):
        def _root():
            if self.iframe:
                try:
                    Page.get_driver().switch_to_default_content()
                    Page.get_driver().switch_to_frame(self.iframe)
                except NoSuchFrameException:
                    holmium.core.log.error("unable to switch to iframe %s" % self.iframe)
            return self.root

        root_function = _root
        for element in inspect.getmembers(self.__class__):
             if issubclass(element[1].__class__, ElementGetter):
                element[1].root_fn = root_function
        return self
    @property
    def root(self):
        return self.__root_val or Page.get_driver().find_element(self.locator_type, self.query_string)

    @root.setter
    def root(self, val):
       self.__root_val = val

class Sections(Section, collections.Sequence):
    """
    Base class for an Iterable view of a collection of :class:`holmium.core.Section`
    objects.
    """
    def __init__(self, locator_type, query_string, iframe=None):
        Section.__init__(self, locator_type, query_string, iframe)
    
    def __getelements__(self):
        return Page.get_driver().find_elements(self.locator_type, self.query_string)
    
    def __iter__(self):
        for el in self.__getelements__():
            self.root = el 
            yield self 
    
    def __len__(self):
        return len(self.__getelements__())
    
    def __getitem__(self, idx):
        _idx = 0 
        for item in self:
            if idx == _idx:
                break 
            _idx+=1 
        if idx > _idx:
            raise IndexError("Sections index (%d) out of range" % idx)
        return self 



import imp
import os
import json
from nose.plugins.base import Plugin
from nose.plugins.skip import SkipTest
import holmium.core
import selenium.webdriver

class HolmiumNose(Plugin):
    """
    nose plugin to allow bootstrapping testcases with a selenium driver
    """
    name = "holmium"
    enabled = False


    def __init__(self):
        Plugin.__init__(self)
        self.driver = None
        self.config = {}
        self.environment = None
        self.driver_initializer_fn = lambda _:None
        self.logger = holmium.core.log

    def options(self, parser, env):
        """
        Register command line options
        """
        parser.add_option("", "--with-holmium", dest="ho_enabled", action="store_true", help="use holmium to setup test case environment")
        parser.add_option("", "--holmium-environment", dest="ho_env", help = "environment to pass to holmium test case configuration")
        parser.add_option("", "--holmium-browser", dest="ho_browser", type = "choice", choices = list(holmium.core.browser_mapping.keys()), help="the selenium driver to invoke")
        parser.add_option("", "--holmium-remote", dest="ho_remote", help = "full url to remote selenium instance")
        parser.add_option("", "--holmium-capabilities", dest="ho_cap", help = "json dictionary of extra capabilities")
        parser.add_option("", "--holmium-useragent", dest="ho_ua", help="User-agent string to use. Only available for firefox & chrome")

    def configure( self, options, conf ):
        if options.ho_enabled:
            self.browser = options.ho_browser or ("HO_BROWSER" in os.environ and os.environ["HO_BROWSER"] )
            self.environment = options.ho_env or ("HO_ENVIRONMENT" in os.environ and os.environ["HO_ENVIRONMENT"])
            self.remote_url = options.ho_remote or ("HO_REMOTE" in os.environ and os.environ["HO_REMOTE"])
            self.user_agent = options.ho_ua or ("HO_USERAGENT" in os.environ and os.environ["HO_USERAGENT"])
            args = {}
            caps = {}
            if self.user_agent:
                if self.browser not in ["chrome","firefox"]:
                    raise SkipTest("useragent string can only be overridden for chrome & firefox")
                else:
                    if self.browser == "chrome":
                        caps.update({"chrome.switches":["--user-agent=%s" % self.user_agent]})
                    elif self.browser == "firefox":
                        ffopts = selenium.webdriver.FirefoxProfile()
                        ffopts.set_preference("general.useragent.override", self.user_agent)
                        ffopts.update_preferences()
                        if options.ho_remote:
                            args.update({"browser_profile":ffopts})
                        else:
                            args.update({"firefox_profile":ffopts})

            if self.remote_url:
                caps.update(holmium.core.capabilities[self.browser])
                args.update( {"command_executor": self.remote_url,
                        "desired_capabilities": caps})
                self.browser = "remote"
            if options.ho_cap:
                try:
                    cap = json.loads( options.ho_cap  )
                    if args.has_key("desired_capabilities"):
                        args["desired_capabilities"].update(cap)
                except Exception as e:
                    self.logger.debug("unable to load capabilities")
                    raise SkipTest("holmium could not be initialized due to a problem with the provided capabilities " + str(e))
            self.driver_initializer_fn = lambda:holmium.core.browser_mapping[self.browser](**args)
            self.enabled = True

    def beforeTest(self, test):
        try:
            if not self.driver:
                self.driver = self.driver_initializer_fn()
        except Exception as e:
            self.logger.exception("failed to initialize selenium driver %s" % e)
            raise SkipTest("holmium could not be initialized due to a problem with the required selenium driver")
        base_file = test.address()[0]
        holmium_vars = { "holmium":
            {
                "environment": self.environment,
                "browser": self.browser,
                "user_agent": self.user_agent,
                "remote": self.remote_url
                }
        }
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        try:
            config = None
            if os.path.isfile(config_path+".json"):
                config = json.loads(open(config_path+".json").read())
            elif os.path.isfile(config_path+".py"):
                config = imp.load_source("config", config_path+".py").config
            if config:
                self.config = holmium.core.Config(config, holmium_vars)
        except Exception as e:
            self.logger.debug("unable to load %s" % config_path)
            raise SkipTest("error in loading config file at path %s" % config_path)

        setattr(test.test, "config", self.config)
    def startTest(self, test):
        if self.driver:
            try:
                setattr(test.test, "driver", self.driver)
                self.driver.delete_all_cookies()
            except Exception as e:
                self.logger.error("error clearing cookies %s" % e)

    def finalize(self, result):
        if self.driver:
            self.driver.quit()




import types
import warnings
import pageobject
import testcase
class Deprecated(object):
    def __new__(cls, *args, **kwargs):
        warnings.warn("The use of %s is deprecated. Use %s instead" % ( cls.cur, cls.alt.__name__)
                , DeprecationWarning
                , stacklevel=2 )
        return super(Deprecated, cls).__new__(cls)

def construct_deprecated(name, alt):
    doc  = """Deprecated alias for :class:`%s`""" % alt.__name__
    cls = types.ClassType(name, (Deprecated, alt), dict(cur=name,alt=alt, __doc__ = doc))
    return cls

PageObject = construct_deprecated("PageObject", pageobject.Page)
PageElement = construct_deprecated("PageElement", pageobject.Element)
PageElements = construct_deprecated("PageElements", pageobject.Elements)
PageElementMap = construct_deprecated("PageElementMap", pageobject.ElementMap)
HolmiumTestCase = construct_deprecated("HolmiumTestCase", testcase.TestCase)

warnings.simplefilter("always")



import os
import unittest
from selenium import webdriver
import inspect
import imp
import holmium
import json
browser_mapping = {"firefox": webdriver.Firefox,
                    "chrome": webdriver.Chrome,
                    "ie": webdriver.Ie,
                    "opera" : webdriver.Opera,
                    "remote": webdriver.Remote,
                    "phantomjs": webdriver.PhantomJS,
                    "iphone" : webdriver.Remote,
                    "ipad": webdriver.Remote,
                    "android": webdriver.Remote}

#:
capabilities = {"firefox": webdriver.DesiredCapabilities.FIREFOX,
                "chrome": webdriver.DesiredCapabilities.CHROME,
                "ie": webdriver.DesiredCapabilities.INTERNETEXPLORER,
                "opera": webdriver.DesiredCapabilities.OPERA,
                "phantomjs":webdriver.DesiredCapabilities.PHANTOMJS,
                "iphone":webdriver.DesiredCapabilities.IPHONE,
                "ipad":webdriver.DesiredCapabilities.IPAD,
                "android":webdriver.DesiredCapabilities.ANDROID}


class TestCase(unittest.TestCase):
    """
    """
    #:
    @classmethod
    def setUp(self):
        """
        """
        pass

    @classmethod
    def setUpClass(self):
        """
        """
        self.driver = None
        base_file = inspect.getfile(self)
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        holmium_vars = { "holmium":
            {
                    "environment": os.environ.get("HO_ENV", "development"),
                    "browser": os.environ.get("HO_BROWSER", "firefox"),
                    "user_agent": os.environ.get("HO_USERAGENT", "").lower(),
                    "remote": os.environ.get("HO_REMOTE", ""),
            }
        }
        config = None
        if os.path.isfile(config_path+".json"):
            config = json.loads(open(config_path+".json").read())
        elif os.path.isfile(config_path+".py"):
            config = imp.load_source("config", config_path+".py").config
        if config:
            self.config = holmium.core.Config(config, holmium_vars)
        args = {}
        cap = {}
        driver = os.environ.get("HO_BROWSER", "firefox").lower()
        remote_url = os.environ.get("HO_REMOTE", "").lower()
        if os.environ.get("HO_USERAGENT", ""):
            if driver not in ["chrome","firefox"]:
                raise SystemExit("useragent string can only be overridden for chrome & firefox")
            else:
                if driver == "chrome":
                    cap.update({"chrome.switches":["--user-agent=%s" % os.environ.get("HO_USERAGENT")]})
                elif driver  == "firefox":
                    ffopts = webdriver.FirefoxProfile()
                    ffopts.set_preference("general.useragent.override", os.environ.get("HO_USERAGENT"))
                    if remote_url:
                        args.update({"browser_profile":ffopts})
                    else:
                        args.update({"firefox_profile":ffopts})
        if remote_url:
            cap.update(capabilities[driver])
            args = {"command_executor": remote_url,
                     "desired_capabilities": cap}
            driver = "remote"
        self.driver = browser_mapping[driver](**args)

    @classmethod
    def tearDownClass(self):
        if self.driver:
            self.driver.quit()

    @classmethod
    def tearDown(self):
        """
        """
        if self.driver:
            self.driver.delete_all_cookies()
